<?xml version="1.0"?>
<!DOCTYPE xsl:stylesheet SYSTEM "xslt.dtd">
<xsl:stylesheet version="1.0"
		xmlns:bibtex="http://www.bitjungle.com/~bibtex/"
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:output method="html" media-type="text/html"/>

  <xsl:template match="/">
    <html>
      <head><title>References</title></head>
      <body>
	<xsl:apply-templates/>
      </body>
    </html>
  </xsl:template>

  <xsl:template match="bibtex:bibliography">
    <h2>References</h2>
    <p>
      <xsl:apply-templates select="bibtex:entry"/>
    </p>
  </xsl:template>

  <xsl:template match="bibtex:entry">
    <span>
      <xsl:attribute name="id">
	<xsl:value-of select="@id"/>
      </xsl:attribute>
      <xsl:apply-templates select="*/bibtex:author"/>
      <xsl:text> (</xsl:text>
      <xsl:value-of select="*/bibtex:year"/>
      <xsl:text>)</xsl:text>
      <xsl:text>. &#xA;</xsl:text>
      <em><xsl:apply-templates select="*/bibtex:title"/></em>
      <xsl:text>. &#xA;</xsl:text>
      <xsl:if test='name(./*)="bibtex:book"'>
	<xsl:value-of select="bibtex:book/bibtex:publisher"/>
	<xsl:text>, </xsl:text>
	<xsl:value-of select="bibtex:book/bibtex:address"/>
	<xsl:text>. &#xA;</xsl:text>
      </xsl:if>
      <xsl:if test='name(./*)="bibtex:article"'>
	<xsl:value-of select="bibtex:article/bibtex:journal"/>
	<xsl:text>, vol.</xsl:text>
	<xsl:value-of select="bibtex:article/bibtex:volume"/>
	<xsl:text>, pp.</xsl:text>
	<xsl:value-of select="bibtex:article/bibtex:pages"/>
	<xsl:text>. &#xA;</xsl:text>
      </xsl:if>
      <xsl:if test='name(./*)="bibtex:techreport"'>
	<xsl:value-of select="bibtex:techreport/bibtex:institution"/>
      </xsl:if>
      <xsl:if test='name(./*)="bibtex:manual"'>
	<xsl:value-of select="bibtex:manual/bibtex:organization"/>
	<xsl:value-of select="*/bibtex:url"/>
	<xsl:value-of select="*/bibtex:uri"/>
      </xsl:if>
    </span>
    <br/>
  </xsl:template>

  <xsl:template match="bibtex:title|bibtex:author">
    <xsl:variable name="url">
      <xsl:text>http://www.amazon.com/exec/obidos/</xsl:text>
      <xsl:value-of select='substring-after(name(.),"bibtex:")'/>
      <xsl:text>=</xsl:text>
      <xsl:value-of select='normalize-space(
		    translate(., " ,:;.&amp;-_", " "))'/>
    </xsl:variable>
    <a>
      <xsl:attribute name="href">
	<xsl:value-of select="$url"/>
      </xsl:attribute>
      <xsl:apply-templates/>
    </a>
  </xsl:template>

</xsl:stylesheet>
